@@ -158,6 +158,8 @@ htp_cfg_t *htp_config_create(void) {
     cfg->extract_request_files = 0;
     cfg->extract_request_files_limit = -1; // Use the parser default.
     cfg->response_decompression_layer_limit = 2; // 2 layers seem fairly common
+    cfg->lzma_memlimit = HTP_LZMA_MEMLIMIT;
+    cfg->compression_bomb_limit = HTP_COMPRESSION_BOMB_LIMIT;
 
     // Default settings for URL-encoded data.
 
@@ -507,6 +509,20 @@ void htp_config_set_field_limits(htp_cfg_t *cfg, size_t soft_limit, size_t hard_
     cfg->field_limit_hard = hard_limit;
 }
 
+void htp_config_set_lzma_memlimit(htp_cfg_t *cfg, size_t memlimit) {
+    if (cfg == NULL) return;
+    cfg->lzma_memlimit = memlimit;
+}
+
+void htp_config_set_compression_bomb_limit(htp_cfg_t *cfg, size_t bomblimit) {
+    if (cfg == NULL) return;
+    if (bomblimit > INT32_MAX) {
+        cfg->compression_bomb_limit = INT32_MAX;
+    } else {
+        cfg->compression_bomb_limit = bomblimit;
+    }
+}
+
 void htp_config_set_log_level(htp_cfg_t *cfg, enum htp_log_level_t log_level) {
     if (cfg == NULL) return;
     cfg->log_level = log_level;
